//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					  (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	OalF90cxxGlue.h	--- the Fortran90/C++ interface of the OAL
//
//	DESCRIPTION:
//		This file acts as the interface between the OAL API written in
//		Fortran90 and its implementation in C++. All platform-specifics
//		of the C++/F90 interface are centralized in this file and
//		OalF90cxxGlue.cc.
//
//	CHANGE HISTORY:
//		UL, 1997-11-25: 0.0.1: - skeleton
//		UL, 1997-??-??: 1.0.0: - first working version
//		UL, 1998-08-20: 2.0.0: - in-line with oal.f90 v2.0.0
//		UL, 1998-09-30: 2.1.0: - new inquiry function OAL_proposalInfo
//		UL, 1998-10-15: 2.2.0: - added fields 'scheduledExposureLength'/
//								 'actualExposureLength' to 'ExposureInfoType'
//							   - added 'isSlew' to 'OdfInfoType'
//							   - new inquiry function 'OAL_activeFilter'
//		UL, 1999-02-03: 2.2.1: - PropoalInfoType.Institute is now 80 chars
//		UL, 1999-02-22: 2.3.0: - dropped fields
//								 'actual/scheduledExposureLength'
//								 corresponding comment in oal.f90
//		UL, 1999-04-01: 2.4.0: - new methods to set state from a reference
//								 to a block or a data set name
//		UL, 1999-06-03: 2.5.0: - removed *n from vector processing routines
//							   - added declaration of wrapper for
//									OAL_ertToTimeTag()
//		UL, 1999-08-18: 2.6.0: - made OAL_getPosition/getAttitude logical
//								 functions
//		UL, 2000-05-31: 2.7.0: - added
//									OAL_hasAssociatedSet()
//									OAL_selectAssociatedSet()
//		UL, 2000-06-15: 2.8.0: - added
//									OAL_hasSet()
//		UL, 2000-09-26: 2.9.0: - OAL_getIPPVstring() is now a function
//							   - added OAL_getIPPVint()
//							   - added OAL_expandSetName()
//		UL, 2001-03-23: 2.10.0:- added OAL_getBadPixelList()
//		UL, 2001-08-10: 2.11.0:- added OAL_selectScopeAndExposure
//							   - added argument 'exposureId' to OAL_setState()
//							   - added arguments 'scheduled'/'exposureNrOdf'
//								 to OAL_getState()
//		UL, 2002-01-07: 2.12.0:- added
//									o OAL_addCommonAttributes()
//							   - renamed 'exposureId' to 'instrExposureId'
//								 as it can now also contain instrument
//								 identifier
//
//	AUTHOR:
//		U. Lammers, Tue Nov 25 16:27:48 MET 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/oal/f90/OalF90cxxGlue.h,v 1.33 2013/01/18 13:57:28 rsaxton Exp $
//

#ifndef OAL_F90CPP_H
#define OAL_F90CPP_H

#include "common.h"
#include "f90cxx.h"
#include "caloaldefs.h"

//
//	---( control routines )-------------------------------------------------
//
extern "C" void CXXFUNC(oal_openodf, OAL_openOdf)
					(const char *directoryName,
					 Integer3 length);


extern "C" void CXXFUNC(oal_setstate1, OAL_setState1)
				(const Integer3 *instrument,
				 const Integer3 *dataMode,
				 const Integer3 *exposureNr,
				 const Logical *scheduled,
				 const Integer3 *ccdNr,
				 const Integer3 *nodeNr,
				 const Logical *attitudeFromAhf,
				 const char *instrExposureId, 
                                 const Integer3 *timeJumpThreshold,
                                       Integer3 length);

extern "C" void CXXFUNC(oal_setstate2, OAL_setState2)
							(char *dataSetName, Integer3 length);

extern "C" void CXXFUNC(oal_setstate3, OAL_setState3)
							(Handle<DataSet> *setP);

extern "C" void CXXFUNC(oal_setstate4, OAL_setState4)
							(Handle<Block> *blockP);

extern "C" void CXXFUNC(oal_setstatefromsetname, OAL_setStateFromSetName)
							(char *setName, Integer3 length);

extern "C" void CXXFUNC(oal_getstate, OAL_getState)
				(Integer3 *instrument,
				 Integer3 *dataMode,
				 Integer3 *exposureNr,
				 Logical *scheduled,
				 Integer3 *ccdNr,
				 Integer3 *nodeNr,
				 Logical *attitudeFromAhf,
				 Integer3 *exposureNrOdf,
                                 Integer3 *timeJumpThreshold);

//
// ---( ODF access )-------------------------------------------------------
//
extern "C" Integer3 CXXFUNC(oal_selectscope, OAL_selectScope)
				(const Integer3 *instrument,
				 const Integer3 *dataMode);

extern "C" Integer3 CXXFUNC(oal_selectexposure, OAL_selectExposure)
				(const Integer3 *exposureNr);

extern "C" Integer3 CXXFUNC(oal_selectscopeandexposure,
							OAL_selectScopeAndExposure)
				(const Integer3 *instrument,
				 const char *exposureId, Integer3 length);

extern "C" void CXXFUNC(oal_selectfile, OAL_selectFile)
				(const Integer3 *ccdNr,
				 char *fileName, Integer3 length);

extern "C" Logical CXXFUNC(oal_hasset, OAL_hasSet)
				(const Integer3 *ccdNr);

extern "C" void CXXFUNC(oal_selectset, OAL_selectSet)
				(char *fileName, Integer3 length,
				 const Integer3 *ccdNr);

extern "C" Logical CXXFUNC(oal_hasassociatedset, OAL_hasAssociatedSet)
				(const Integer3 *dataMode);

extern "C" void CXXFUNC(oal_selectassociatedset, OAL_selectAssociatedSet)
				(char *setName, Integer3 length,
				 const Integer3 *dataMode);

extern "C" void CXXFUNC(oal_expandfilename, OAL_expandFileName)
				(const char *inFileName, char *outFileName,
				 Integer3 lengthInFileName, Integer3 lengthOutFileName);

extern "C" void CXXFUNC(oal_expandsetname, OAL_expandSetName)
				(char *setName, Integer3 length,
				 const char *inFileName, Integer3 lengthInFileName);

extern "C" void CXXFUNC(oal_framecountertoobt, OAL_frameCounterToObt)
				(const Dope1 *frameCounters,
				 Dope1 *obts);

extern "C" void CXXFUNC(oal_obttotimetag, OAL_obtToTimeTag)
				(const Dope1 *obts,
				 Dope1 *timeTags);

extern "C" Double CXXFUNC(oal_erttotimetag, OAL_ertToTimeTag)
				(const char *ertString, Integer3 length);

extern "C" Logical CXXFUNC(oal_getattitude, OAL_getAttitude)
							(const Double *timeTag,
							 SpacecraftAttitudeType *attitude);

typedef struct { // must match definitions of
				 // CartesianVector type in oal.f90
	Double	x, y, z;
} CartesianVectorType;

extern "C" Logical CXXFUNC(oal_getposition, OAL_getPosition)
							(const Double *timeTag,
							 CartesianVectorType *position,
							 CartesianVectorType *velocity,
							 Double *revolution);

extern "C" Logical CXXFUNC(oal_getbadpixellist, OAL_getBadPixelList)
							(Dope1 *rawX,
							 Dope1 *rawY,
							 Dope1 *type,
							 Dope1 *yextent,
							 Dope1 *uplinked);

//
//	---( inquiry / utility functions )---------------------------------------
//
typedef struct { // must match definitions of OdfInfoType type
                 // in oal.f90
	Logical  isSlew;			// Is this a slew ODF?
	Integer3 revolutionNr,		// revolution (orbit) number
			nrFile;				// number of files this ODF consists of
	char	proposalId[10],		// Id of proposal this ODF belongs to
			observationId[10];	// observation Id

	char	obsStartTime[25],	// observation start time yyyy-mm-ddThh:mm:ss
			obsEndTime[25];		// observation end time yyyy-mm-ddThh:mm:ss

	char	directoryName[1024];// where is the ODF on the local file system
} OdfInfoType;

extern "C" void CXXFUNC(oal_odfinfo, OAL_odfInfo)
				(OdfInfoType *odfInfo);


typedef struct { // must match definitions of ProposalInfoType type
				 // in oal.f90
	char		GoName[80],			// name of GO
				GoInstitute[80],	// his institute
				GoAddress[120],		// postal address
				GoEmail[80],		// e-mail address
				AO[2],				// AO
				scienceType[2],		// science Type
				targetName[20];		// name of target
	Double		targetRa, targetDec,// target coordinates
				obsDuration,		// duration of observation in sec
				boresightRa,		// boresight Ra
				boresightDec;		// boresight Dec
	char		posAngleConstLo[3],
				posAngleConstHi[3],	// position angle constraints
				posAngleOrigRef[2];
	Integer2	emos1Prio, emos2Prio, epnPrio, rgs1Prio, rgs2Prio, omPrio;
									// instrument priorities
} ProposalInfoType;

extern "C" void CXXFUNC(oal_proposalinfo, OAL_proposalInfo)
				(ProposalInfoType *proposalInfo);


typedef struct { // must match definitions of ExposureInfoType type
                 // in oal.f90
	Integer3	number;			// exposure sequence number
	Logical		scheduled;		// was exposure scheduled?
	Integer3	type;			// 0=science, 1=calibration

	char		scheduledStartTime[25],
				scheduledEndTime[25],
				actualStartTime[25],
				actualEndTime[25];
					// format: yyyy-mm-ddThh:mm:ss
} ExposureInfoType;

extern "C" void CXXFUNC(oal_exposureinfo, OAL_exposureInfo)
				(ExposureInfoType *exposureInfo);

extern "C" Logical CXXFUNC(oal_hasippv, OAL_hasIPPV)
				(const char *keyword, Integer3 length);

extern "C" Integer3 CXXFUNC(oal_getippvint, OAL_getIPPVint)
                (const char *keyword, Integer3 length);

extern "C" Double CXXFUNC(oal_getippvreal, OAL_getIPPVreal)
                (const char *keyword, Integer3 length);

extern "C" void CXXFUNC(oal_getippvstring, OAL_getIPPVstring)
				(char *value, const Integer3 valueLength,
				 const char *keyword, Integer3 keyLength);

extern "C" Logical CXXFUNC(oal_hasconfippv, OAL_hasIPPV)
                (const char *keyword, Integer3 length);
                                                                                
extern "C" Integer3 CXXFUNC(oal_getconfippvint, OAL_getIPPVint)
                (const char *keyword, Integer3 length);
                                                                                
extern "C" Double CXXFUNC(oal_getconfippvreal, OAL_getIPPVreal)                 (const char *keyword, Integer3 length);
                                                                                
extern "C" void CXXFUNC(oal_getconfippvstring, OAL_getIPPVstring)
                (char *value, const Integer3 valueLength,
                 const char *keyword, Integer3 keyLength);
                                                                                
extern "C" void CXXFUNC(oal_toattitudematrix, OAL_toAttitudeMatrix)
				(const SpacecraftAttitudeType *attitude,
				 Double *matrix);

typedef struct { // must match definitions of
				 // EulerAnglesType in oal.f90
	Double phi, theta, psi;
} EulerAngs;

extern "C" void CXXFUNC(oal_toeulerangles, OAL_toEulerAngles)
				(const Dope2 *matrix,
				 EulerAngs *eulerAngles);

extern "C" void CXXFUNC(oal_addcommonattributes, OAL_addCommonAttributes)
				(Handle<DataSet> *setP);

//
//	---( release memory )-------------------------------------------------
//
#define RELEASEPROTO(fname, cname, N)		\
	extern "C" void CXXFUNC(fname, cname)	\
		(Dope##N *a1, Dope##N *a2, Dope##N *a3)

RELEASEPROTO(oal_releasememoryreal2onedim, OAL_releaseMemoryReal2OneDim, 1);
RELEASEPROTO(oal_releasememoryint2onedim, OAL_releaseMemoryInt2OneDim, 1);
RELEASEPROTO(oal_releasememorylogicalonedim, OAL_releaseMemoryLogicalOneDim,1);

#undef RELEASEPROTO

#endif
